@@ -4,7 +4,8 @@ logger = logging.getLogger(__name__)
 from django.db import models
 from django.db.models import permalink
 
-from django.template import Template, Context
+from django.template import Context, TemplateDoesNotExist
+from django.template.loader import select_template
 
 from django.utils.translation import ugettext_lazy as _
 from django.utils.translation import ugettext
@@ -21,82 +22,7 @@ from django.conf import settings
 
 from sorl.thumbnail import ImageField
 
-from .utils import make_activation_code, get_default_sites
-
-
-class EmailTemplate(models.Model):
-    """ Model we use to store email (Django) templates in the database. """
-
-    ACTION_CHOICES = (
-        ('subscribe', _('Subscribe')),
-        ('unsubscribe', _('Unsubscribe')),
-        ('update', _('Update')),
-        ('message', _('Message')),
-    )
-
-    def __unicode__(self):
-        return u"%s '%s'" % (self.get_action_display(), self.title)
-
-    @classmethod
-    def get_templates(cls, action, newsletter):
-        assert action in ['subscribe', 'unsubscribe', 'update', 'message'], \
-            'Unknown action %s' % action
-
-        myemail = getattr(newsletter, '%s_template' % action)
-
-        if myemail.html:
-            # If HTML available, return (subject, text, html) tuple
-            return (
-                Template(myemail.subject),
-                Template(myemail.text),
-                Template(myemail.html)
-            )
-        else:
-            return (Template(myemail.subject), Template(myemail.text), None)
-
-    class Meta:
-        verbose_name = _('e-mail template')
-        verbose_name_plural = _('e-mail templates')
-
-        unique_together = ("title", "action")
-        ordering = ('title', )
-
-    @classmethod
-    def get_default_id(cls, action):
-        try:
-            ls = EmailTemplate.objects.filter(action__exact=action)
-            if ls.count() == 1:
-                return ls[0].id
-            else:
-                ls = ls.filter(title__exact=_('Default'))
-                if ls.count():
-                    #There can be only one of these
-                    return ls[0].id
-        except:
-            pass
-
-        return None
-
-    title = models.CharField(
-        max_length=200, verbose_name=_('name'), default=_('Default')
-    )
-    action = models.CharField(
-        max_length=16, choices=ACTION_CHOICES, db_index=True,
-        verbose_name=_('action')
-    )
-
-    subject = models.CharField(max_length=255, verbose_name=_('subject'))
-
-    text = models.TextField(
-        verbose_name=_('Text'),
-        help_text=_('Plain text e-mail message. Available objects: date, '
-                    'subscription, site, submission, newsletter, STATIC_URL, '
-                    'MEDIA_URL and message.')
-    )
-    html = models.TextField(
-        verbose_name=_('HTML'), help_text=_('HTML e-mail alternative.'),
-        null=True, blank=True
-    )
+from .utils import make_activation_code, get_default_sites, ACTIONS
 
 
 class Newsletter(models.Model):
@@ -118,38 +44,54 @@ class Newsletter(models.Model):
         default=True, verbose_name=_('visible'), db_index=True
     )
 
+    send_html = models.BooleanField(
+        default=True, verbose_name=_('send html'),
+        help_text=_('Whether or not to send HTML versions of e-mails.')
+    )
+
     objects = models.Manager()
 
     # Automatically filter the current site
     on_site = CurrentSiteManager()
 
-    subscribe_template = models.ForeignKey(
-        'EmailTemplate',
-        default=lambda: EmailTemplate.get_default_id('subscribe'),
-        related_name='subcribe_template', verbose_name=_('subscribe template'),
-        limit_choices_to={'action': 'subscribe'}
-    )
-    unsubscribe_template = models.ForeignKey(
-        'EmailTemplate',
-        default=lambda: EmailTemplate.get_default_id('unsubscribe'),
-        related_name='unsubcribe_template',
-        verbose_name=_('unsubscribe template'),
-        limit_choices_to={'action': 'unsubscribe'}
-    )
-    update_template = models.ForeignKey(
-        'EmailTemplate',
-        default=lambda: EmailTemplate.get_default_id('update'),
-        related_name='update_template',
-        verbose_name=_('update template'),
-        limit_choices_to={'action': 'update'}
-    )
-    message_template = models.ForeignKey(
-        'EmailTemplate',
-        default=lambda: EmailTemplate.get_default_id('message'),
-        related_name='message_template',
-        verbose_name=_('message template'),
-        limit_choices_to={'action': 'message'}
-    )
+    def get_templates(self, action):
+        """
+        Return a subject, text, HTML tuple with e-mail templates for
+        a particular action. Returns a tuple with subject, text and e-mail
+        template.
+        """
+
+        assert action in ACTIONS + ('message', ), 'Unknown action: %s' % action
+
+        # Common substitutions for filenames
+        tpl_subst = {
+            'action': action,
+            'newsletter': self.slug
+        }
+
+        # Common root path for all the templates
+        tpl_root = 'newsletter/message/'
+
+        subject_template = select_template([
+            tpl_root + '%(newsletter)s/%(action)s_subject.txt' % tpl_subst,
+            tpl_root + '%(action)s_subject.txt' % tpl_subst,
+        ])
+
+        text_template = select_template([
+            tpl_root + '%(newsletter)s/%(action)s.txt' % tpl_subst,
+            tpl_root + '%(action)s.txt' % tpl_subst,
+        ])
+
+        if self.send_html:
+            html_template = select_template([
+                tpl_root + '%(newsletter)s/%(action)s.html' % tpl_subst,
+                tpl_root + '%(action)s.html' % tpl_subst,
+            ])
+        else:
+            # HTML templates are not required
+            html_template = None
+
+        return (subject_template, text_template, html_template)
 
     def __unicode__(self):
         return self.title
@@ -247,14 +189,49 @@ class Subscription(models.Model):
             self.email_field = email
     email = property(get_email, set_email)
 
-    def subscribe(self):
+    def update(self, action):
+        """
+        Update subscription according to requested action:
+        subscribe/unsubscribe/update/, then save the changes.
+        """
+
+        assert action in ('subscribe', 'update', 'unsubscribe')
+
+        # If a new subscription or update, make sure it is subscribed
+        # Else, unsubscribe
+        if action == 'subscribe' or action == 'update':
+            self.subscribed = True
+        else:
+            self.unsubscribed = True
+
+        logger.debug(
+            _(u'Updated subscription %(subscription)s to %(action)s.'),
+            {
+                'subscription': self,
+                'action': action
+            }
+        )
+
+        # This triggers the subscribe() and/or unsubscribe() methods, taking
+        # care of stuff like maintaining the (un)subscribe date.
+        self.save()
+
+    def _subscribe(self):
+        """
+        Internal helper method for managing subscription state
+        during subscription.
+        """
         logger.debug(u'Subscribing subscription %s.', self)
 
         self.subscribe_date = now()
         self.subscribed = True
         self.unsubscribed = False
 
-    def unsubscribe(self):
+    def _unsubscribe(self):
+        """
+        Internal helper method for managing subscription state
+        during unsubscription.
+        """
         logger.debug(u'Unsubscribing subscription %s.', self)
 
         self.subscribed = False
@@ -262,6 +239,13 @@ class Subscription(models.Model):
         self.unsubscribe_date = now()
 
     def save(self, *args, **kwargs):
+        """
+        Perform some basic validation and state maintenance of Subscription.
+
+        TODO: Move this code to a more suitable place (i.e. `clean()`) and
+        cleanup the code. Refer to comment below and
+        https://docs.djangoproject.com/en/dev/ref/models/instances/#django.db.models.Model.clean
+        """
         assert self.user or self.email_field, \
             _('Neither an email nor a username is set. This asks for '
               'inconsistency!')
@@ -287,7 +271,7 @@ class Subscription(models.Model):
             # If we user to be unsubscribed but are not so anymore, subscribe.
             if ((self.subscribed and not old_subscribed) or
                (old_unsubscribed and not self.unsubscribed)):
-                self.subscribe()
+                self._subscribe()
 
                 assert not self.unsubscribed
                 assert self.subscribed
@@ -297,15 +281,15 @@ class Subscription(models.Model):
             # unsubscribe.
             elif ((self.unsubscribed and not old_unsubscribed) or
                   (old_subscribed and not self.subscribed)):
-                self.unsubscribe()
+                self._unsubscribe()
 
                 assert not self.subscribed
                 assert self.unsubscribed
         else:
             if self.subscribed:
-                self.subscribe()
+                self._subscribe()
             elif self.unsubscribed:
-                self.unsubscribe()
+                self._unsubscribe()
 
         super(Subscription, self).save(*args, **kwargs)
 
@@ -361,11 +345,10 @@ class Subscription(models.Model):
         return u'%s' % (self.email)
 
     def send_activation_email(self, action):
-        assert action in ['subscribe', 'unsubscribe', 'update'], \
-            'Unknown action'
+        assert action in ACTIONS, 'Unknown action: %s' % action
 
         (subject_template, text_template, html_template) = \
-            EmailTemplate.get_templates(action, self.newsletter)
+            self.newsletter.get_templates(action)
 
         variable_dict = {
             'subscription': self,
@@ -378,9 +361,11 @@ class Subscription(models.Model):
 
         unescaped_context = Context(variable_dict, autoescape=False)
 
+        subject = subject_template.render(unescaped_context).strip()
+        text = text_template.render(unescaped_context)
+
         message = EmailMultiAlternatives(
-            subject_template.render(unescaped_context),
-            text_template.render(unescaped_context),
+            subject, text,
             from_email=self.newsletter.get_sender(),
             to=[self.email]
         )
@@ -388,8 +373,9 @@ class Subscription(models.Model):
         if html_template:
             escaped_context = Context(variable_dict)
 
-            message.attach_alternative(html_template.render(escaped_context),
-                                      "text/html")
+            message.attach_alternative(
+                html_template.render(escaped_context), "text/html"
+            )
 
         message.send()
 
@@ -398,7 +384,8 @@ class Subscription(models.Model):
             u'with activation code "%(action_code)s".', {
                 'action_code': self.activation_code,
                 'action': action,
-                'subscriber': self}
+                'subscriber': self
+            }
         )
 
     @permalink
@@ -513,7 +500,7 @@ class Message(models.Model):
     class Meta:
         verbose_name = _('message')
         verbose_name_plural = _('messages')
-        unique_together = ("slug", "newsletter")
+        unique_together = ('slug', 'newsletter')
 
     @classmethod
     def get_default_id(cls):
@@ -559,7 +546,7 @@ class Submission(models.Model):
 
         try:
             (subject_template, text_template, html_template) = \
-                EmailTemplate.get_templates('message', self.message.newsletter)
+                self.message.newsletter.get_templates('message')
 
             for subscription in subscriptions:
                 variable_dict = {
@@ -575,9 +562,11 @@ class Submission(models.Model):
 
                 unescaped_context = Context(variable_dict, autoescape=False)
 
+                subject = subject_template.render(unescaped_context).strip()
+                text = text_template.render(unescaped_context)
+
                 message = EmailMultiAlternatives(
-                    subject_template.render(unescaped_context),
-                    text_template.render(unescaped_context),
+                    subject, text,
                     from_email=self.newsletter.get_sender(),
                     to=[subscription.get_recipient()]
                 )
