@@ -2,24 +2,29 @@
 # Get the with statement from the future
 from __future__ import with_statement
 
-from datetime import timedelta
+from datetime import datetime, timedelta
 
 import time
 
+# Conditioally import pytz
+try:
+    import pytz
+except ImportError:
+    pytz = None
+
 from django import VERSION as DJANGO_VERSION
 
 from django.core import mail
 from django.core.urlresolvers import reverse
 
-from django.utils.timezone import now
+from django.utils import unittest, timezone
 
 from django.test.utils import override_settings
 
 from ..models import (
-    Newsletter, Subscription, Submission, Message,
-    EmailTemplate, get_default_sites
+    Newsletter, Subscription, Submission, Message, get_default_sites
 )
-from ..forms import SubscribeRequestForm, UpdateForm, UpdateRequestForm
+from ..forms import UpdateForm
 
 from .utils import MailTestCase, UserTestCase, WebTestCase, ComparingTestCase
 
@@ -31,7 +36,7 @@ WAIT_TIME = 1
 class NewsletterListTestCase(WebTestCase):
     """ Base class for newsletter test cases. """
 
-    fixtures = ['default_templates', 'test_newsletters']
+    fixtures = ['test_newsletters']
 
     def setUp(self):
         self.newsletters = Newsletter.objects.all()
@@ -297,9 +302,7 @@ class UserNewsletterListTestCase(UserTestCase,
             Subscription.objects.filter(subscribed=True).exists())
 
 
-class WebSubscribeTestCase(WebTestCase, MailTestCase):
-
-    fixtures = ['default_templates']
+class SubscribeTestCase(WebTestCase, MailTestCase):
 
     def setUp(self):
         self.n = Newsletter(title='Test newsletter',
@@ -312,36 +315,68 @@ class WebSubscribeTestCase(WebTestCase, MailTestCase):
         self.subscribe_url = \
             reverse('newsletter_subscribe_request',
                     kwargs={'newsletter_slug': self.n.slug})
-
         self.subscribe_confirm_url = \
             reverse('newsletter_subscribe_confirm',
                     kwargs={'newsletter_slug': self.n.slug})
+        self.subscribe_email_sent_url = \
+            reverse('newsletter_activation_email_sent',
+                    kwargs={'newsletter_slug': self.n.slug,
+                            'action': 'subscribe'})
+        self.subscribe_activated_url = \
+            reverse('newsletter_action_activated',
+                    kwargs={'newsletter_slug': self.n.slug,
+                            'action': 'subscribe'})
 
         self.update_url = \
             reverse('newsletter_update_request',
                     kwargs={'newsletter_slug': self.n.slug})
+        self.update_email_sent_url = \
+            reverse('newsletter_activation_email_sent',
+                    kwargs={'newsletter_slug': self.n.slug,
+                            'action': 'update'})
+        self.update_activated_url = \
+            reverse('newsletter_action_activated',
+                    kwargs={'newsletter_slug': self.n.slug,
+                            'action': 'update'})
 
         self.unsubscribe_url = \
             reverse('newsletter_unsubscribe_request',
                     kwargs={'newsletter_slug': self.n.slug})
-
         self.unsubscribe_confirm_url = \
             reverse('newsletter_unsubscribe_confirm',
                     kwargs={'newsletter_slug': self.n.slug})
+        self.unsubscribe_email_sent_url = \
+            reverse('newsletter_activation_email_sent',
+                    kwargs={'newsletter_slug': self.n.slug,
+                            'action': 'unsubscribe'})
+        self.unsubscribe_activated_url = \
+            reverse('newsletter_action_activated',
+                    kwargs={'newsletter_slug': self.n.slug,
+                            'action': 'unsubscribe'})
 
-        super(WebSubscribeTestCase, self).setUp()
+        super(SubscribeTestCase, self).setUp()
 
     def test_urls(self):
+        # TODO: is performing this test in each subclass
+        #     of WebSubscribeTestCase really needed?
         self.assert_(self.subscribe_url)
         self.assert_(self.update_url)
         self.assert_(self.unsubscribe_url)
         self.assert_(self.subscribe_confirm_url)
         self.assert_(self.unsubscribe_confirm_url)
+        self.assert_(self.subscribe_email_sent_url)
+        self.assert_(self.update_email_sent_url)
+        self.assert_(self.unsubscribe_email_sent_url)
+        self.assert_(self.subscribe_activated_url)
+        self.assert_(self.update_activated_url)
+        self.assert_(self.unsubscribe_activated_url)
 
 
-class WebUserSubscribeTestCase(WebSubscribeTestCase,
-                               UserTestCase,
-                               ComparingTestCase):
+class UserSubscribeTestCase(
+    SubscribeTestCase,
+    UserTestCase,
+    ComparingTestCase
+):
     """ Test case for user subscription and unsubscription."""
 
     def get_user_subscription(self):
@@ -412,7 +447,7 @@ class WebUserSubscribeTestCase(WebSubscribeTestCase,
         subscription.save()
 
         self.assertLessThan(
-            subscription.subscribe_date, now() + timedelta(seconds=1)
+            subscription.subscribe_date, timezone.now() + timedelta(seconds=1)
         )
 
         response = self.client.get(self.unsubscribe_url)
@@ -461,7 +496,7 @@ class WebUserSubscribeTestCase(WebSubscribeTestCase,
         self.assert_(subscription.unsubscribed)
         self.assertLessThan(
             subscription.unsubscribe_date,
-            now() + timedelta(seconds=1)
+            timezone.now() + timedelta(seconds=1)
         )
 
     def test_unsubscribe_twice(self):
@@ -484,8 +519,22 @@ class WebUserSubscribeTestCase(WebSubscribeTestCase,
         self.assertNotContains(response, 'id="id_submit"')
 
 
-class AnonymousSubscribeTestCase(WebSubscribeTestCase,
-                                 ComparingTestCase):
+class AnonymousSubscribeTestCase(
+    SubscribeTestCase,
+    ComparingTestCase
+):
+
+    def get_only_subscription(self, **kwargs):
+        """
+        Assert there's exactly one subscription that match kwargs lookup.
+        Return this subscription.
+        """
+
+        subscription_qs = self.n.subscription_set.filter(**kwargs)
+
+        self.assertEquals(subscription_qs.count(), 1)
+
+        return subscription_qs[0]
 
     def test_subscribe_request_view(self):
         """ Test the subscription form. """
@@ -499,6 +548,7 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
 
         self.assertEqual(response.context['newsletter'], self.n)
 
+    @override_settings(NEWSLETTER_CONFIRM_EMAIL_SUBSCRIBE=True)
     def test_subscribe_request_post(self):
         """ Post the subscription form. """
 
@@ -509,20 +559,13 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
             }
         )
 
-        self.assertContains(response, self.n.title, status_code=200)
-        self.assertNotContains(
-            response, 'input id="id_name_field" type="text" name="name"'
-        )
-        self.assertNotContains(
-            response, 'input id="id_email_field" type="text" name="email"'
+        # Assure we are redirected to "subscribe activation email sent" page.
+        self.assertRedirects(response, self.subscribe_email_sent_url)
+
+        subscription = self.get_only_subscription(
+            email_field__exact='test@email.com'
         )
 
-        self.assertInContext(response, 'newsletter', Newsletter, self.n)
-        self.assertInContext(response, 'form', SubscribeRequestForm)
-        self.assertFalse(response.context['error'])
-
-        subscription = getattr(response.context['form'], 'instance', None)
-        self.assert_(subscription)
         self.assertFalse(subscription.subscribed)
         self.assertFalse(subscription.unsubscribed)
 
@@ -534,8 +577,39 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
 
         self.assertEmailContains(full_activate_url)
 
+    @override_settings(NEWSLETTER_CONFIRM_EMAIL_SUBSCRIBE=False)
+    def test_subscribe_request_post_no_email(self):
+        """
+        Post the subscription form
+        with confirmation email switched off in settings.
+        """
+
+        response = self.client.post(
+            self.subscribe_url, {
+                'name_field': 'Test Name',
+                'email_field': 'test@email.com'
+            }
+        )
+
+        # Assure we are redirected to "subscribe activated" page.
+        self.assertRedirects(response, self.subscribe_activated_url)
+
+        subscription = self.get_only_subscription(
+            email_field__exact='test@email.com'
+        )
+
+        # email confirmation is switched off,
+        # so after subscribe request user should be subscribed
+        self.assertTrue(subscription.subscribed)
+        self.assertFalse(subscription.unsubscribed)
+
+        """ Check the subscription email. """
+        # no email should be send
+        self.assertEquals(len(mail.outbox), 0)
+
     # Only run this test when settings overrides are available
-    def test_subscrube_request_post_error(self):
+    @override_settings(NEWSLETTER_CONFIRM_EMAIL_SUBSCRIBE=True)
+    def test_subscribe_request_post_error(self):
         """
         Test whether a failing subscribe request email generated an error in
         the context.
@@ -544,7 +618,9 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
         we know pretty sure is bound to fail.
         """
 
-        with override_settings(EMAIL_BACKEND='django.core.mail.backends.smtp.EmailBackend'):
+        with override_settings(
+            EMAIL_BACKEND='django.core.mail.backends.smtp.EmailBackend'
+        ):
             with override_settings(EMAIL_PORT=12345678):
                 response = self.client.post(
                     self.subscribe_url, {
@@ -630,11 +706,14 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
             }
         )
 
-        self.assertEquals(response.status_code, 200)
+        # Assure we are redirected to "unsubscribe activated" page.
+        self.assertRedirects(response, self.unsubscribe_activated_url)
 
-        subscription = getattr(response.context['form'], 'instance', None)
+        subscription = self.get_only_subscription(
+            email_field__exact='test@email.com'
+        )
 
-        # self.assertFalse(subscription.subscribed)
+        self.assertFalse(subscription.subscribed)
         self.assert_(subscription.unsubscribed)
 
         # Resubscribe request
@@ -646,7 +725,8 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
             }
         )
 
-        self.assertEquals(response.status_code, 200)
+        # Assure we are redirected to "email sent page"
+        self.assertRedirects(response, self.subscribe_email_sent_url)
 
         # self.assertFalse(subscription.subscribed)
         self.assert_(subscription.unsubscribed)
@@ -660,9 +740,14 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
                 'user_activation_code': subscription.activation_code
             }
         )
-        self.assertInContext(response, 'form', UpdateForm)
 
-        subscription = getattr(response.context['form'], 'instance', None)
+        # Assure we are redirected to "subscribe activated" page.
+        self.assertRedirects(response, self.subscribe_activated_url)
+
+        subscription = self.get_only_subscription(
+            email_field__exact='test@email.com'
+        )
+
         self.assert_(subscription.subscribed)
         self.assertFalse(subscription.unsubscribed)
 
@@ -719,16 +804,21 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
                 'user_activation_code': subscription.activation_code
             }
         )
-        self.assertInContext(response, 'form', UpdateForm)
 
-        subscription = getattr(response.context['form'], 'instance', None)
-        self.assert_(subscription)
+        # Assure we are redirected to "subscribe activated" page.
+        self.assertRedirects(response, self.subscribe_activated_url)
+
+        subscription = self.get_only_subscription(
+            email_field__exact='test@email.com'
+        )
+
         self.assert_(subscription.subscribed)
         self.assertFalse(subscription.unsubscribed)
 
         dt = (subscription.subscribe_date - subscription.create_date).seconds
         self.assertBetween(dt, WAIT_TIME, WAIT_TIME + 1)
 
+    @override_settings(NEWSLETTER_CONFIRM_EMAIL_UNSUBSCRIBE=True)
     def test_unsubscribe_request_post(self):
         """ Post the unsubscribe request form. """
 
@@ -742,16 +832,8 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
             self.unsubscribe_url, {'email_field': 'test@email.com'}
         )
 
-        self.assertContains(response, self.n.title, status_code=200)
-        self.assertNotContains(
-            response, 'input id="id_email_field" type="text" name="email"'
-        )
-
-        self.assertInContext(response, 'newsletter', Newsletter, self.n)
-        self.assertInContext(response, 'form', UpdateRequestForm)
-        self.assertFalse(response.context['error'])
-
-        self.assertEqual(subscription, response.context['form'].instance)
+        # Assure we are redirected to "unsubscribe activation email sent" page.
+        self.assertRedirects(response, self.unsubscribe_email_sent_url)
 
         """ Check the subscription email. """
         self.assertEquals(len(mail.outbox), 1)
@@ -761,6 +843,40 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
 
         self.assertEmailContains(full_activate_url)
 
+    @override_settings(NEWSLETTER_CONFIRM_EMAIL_UNSUBSCRIBE=False)
+    def test_unsubscribe_request_post_no_email(self):
+        """
+        Post the unsubscribe request form
+        with confirmation email switched off in settings.
+        """
+
+        subscription = Subscription(newsletter=self.n,
+                                    name='Test Name',
+                                    email='test@email.com',
+                                    subscribed=True)
+        subscription.save()
+
+        response = self.client.post(
+            self.unsubscribe_url, {'email_field': 'test@email.com'}
+        )
+
+        # Assure we are redirected to "unsubscribe activated" page.
+        self.assertRedirects(response, self.unsubscribe_activated_url)
+
+        changed_subscription = self.get_only_subscription(
+            email_field__exact='test@email.com'
+        )
+
+        # email confirmation is switched off,
+        # so after unsubscribe request user should be unsubscribed
+        self.assertFalse(changed_subscription.subscribed)
+        self.assertTrue(changed_subscription.unsubscribed)
+
+        """ Check the subscription email. """
+        # no email should be send
+        self.assertEquals(len(mail.outbox), 0)
+
+    @override_settings(NEWSLETTER_CONFIRM_EMAIL_UNSUBSCRIBE=True)
     def test_unsubscribe_request_post_error(self):
         """
         Test whether a failing unsubscribe request email generated an error in
@@ -775,7 +891,9 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
                                     subscribed=True)
         subscription.save()
 
-        with override_settings(EMAIL_BACKEND='django.core.mail.backends.smtp.EmailBackend'):
+        with override_settings(
+            EMAIL_BACKEND='django.core.mail.backends.smtp.EmailBackend'
+        ):
             with override_settings(EMAIL_PORT=12345678):
                 response = self.client.post(
                     self.unsubscribe_url, {'email_field': 'test@email.com'}
@@ -814,15 +932,19 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
             'email_field': testemail2,
             'user_activation_code': subscription.activation_code
         })
-        self.assertInContext(response, 'form', UpdateForm)
 
-        subscription = getattr(response.context['form'], 'instance', None)
-        self.assert_(subscription)
+        # Assure we are redirected to "unsubscribe activated" page.
+        self.assertRedirects(response, self.unsubscribe_activated_url)
+
+        subscription = self.get_only_subscription(
+            email_field__exact=testemail2
+        )
+
         self.assert_(subscription.unsubscribed)
         self.assertEqual(subscription.name, testname2)
         self.assertEqual(subscription.email, testemail2)
 
-        dt = (now() - subscription.unsubscribe_date).seconds
+        dt = (timezone.now() - subscription.unsubscribe_date).seconds
         self.assertLessThan(dt, 2)
 
     def test_update_request_view(self):
@@ -835,6 +957,7 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
 
         self.assertEqual(response.context['newsletter'], self.n)
 
+    @override_settings(NEWSLETTER_CONFIRM_EMAIL_UPDATE=True)
     def test_update_request_post(self):
         """ Test the update request post view. """
 
@@ -848,16 +971,8 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
             self.update_url, {'email_field': 'test@email.com'}
         )
 
-        self.assertContains(response, self.n.title, status_code=200)
-        self.assertNotContains(
-            response, 'input id="id_email_field" type="text" name="email"'
-        )
-
-        self.assertInContext(response, 'newsletter', Newsletter, self.n)
-        self.assertInContext(response, 'form', UpdateRequestForm)
-        self.assertFalse(response.context['error'])
-
-        self.assertEqual(subscription, response.context['form'].instance)
+        # Assure we are redirected to "update activation email sent" page.
+        self.assertRedirects(response, self.update_email_sent_url)
 
         """ Check the subscription email. """
         self.assertEquals(len(mail.outbox), 1)
@@ -867,6 +982,30 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
 
         self.assertEmailContains(full_activate_url)
 
+    @override_settings(NEWSLETTER_CONFIRM_EMAIL_UPDATE=False)
+    def test_update_request_post_no_email(self):
+        """
+        Test the update request post view
+        with confirmation email switched off in settings.
+        """
+
+        subscription = Subscription(newsletter=self.n,
+                                    name='Test Name',
+                                    email='test@email.com',
+                                    subscribed=True)
+        subscription.save()
+
+        response = self.client.post(
+            self.update_url, {'email_field': 'test@email.com'}
+        )
+
+        self.assertRedirects(response, subscription.update_activate_url())
+
+        """ Check the subscription email. """
+        # no email should be send
+        self.assertEquals(len(mail.outbox), 0)
+
+    @override_settings(NEWSLETTER_CONFIRM_EMAIL_UPDATE=True)
     def test_update_request_post_error(self):
         """
         Test whether a failing update request email generated an error in
@@ -881,7 +1020,9 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
                                     subscribed=True)
         subscription.save()
 
-        with override_settings(EMAIL_BACKEND='django.core.mail.backends.smtp.EmailBackend'):
+        with override_settings(
+            EMAIL_BACKEND='django.core.mail.backends.smtp.EmailBackend'
+        ):
             with override_settings(EMAIL_PORT=12345678):
                 response = self.client.post(
                     self.update_url, {'email_field': 'test@email.com'}
@@ -899,7 +1040,9 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
         subscription.save()
 
         for url in (self.update_url, self.unsubscribe_url):
-            response = self.client.post(url, {'email_field': 'test@email.com'})
+            response = self.client.post(
+                url, {'email_field': 'test@email.com'}
+            )
 
             self.assertContains(
                 response, "This subscription has not yet been activated."
@@ -926,7 +1069,7 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
                                     email='test@email.com')
         subscription.save()
 
-        activate_url = subscription.subscribe_activate_url()
+        activate_url = subscription.update_activate_url()
 
         response = self.client.get(activate_url)
         self.assertInContext(response, 'form', UpdateForm)
@@ -939,9 +1082,14 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
             'email_field': testemail2,
             'user_activation_code': subscription.activation_code
         })
-        self.assertInContext(response, 'form', UpdateForm)
 
-        subscription = getattr(response.context['form'], 'instance', None)
+        # Assure we are redirected to "update activated" page.
+        self.assertRedirects(response, self.update_activated_url)
+
+        subscription = self.get_only_subscription(
+            email_field__exact=testemail2
+        )
+
         self.assert_(subscription)
         self.assert_(subscription.subscribed)
         self.assertEqual(subscription.name, testname2)
@@ -970,6 +1118,43 @@ class AnonymousSubscribeTestCase(WebSubscribeTestCase,
         self.assertInContext(response, 'form', UpdateForm)
 
 
+class InvisibleAnonymousSubscribeTestCase(AnonymousSubscribeTestCase):
+    """
+    Tests for subscribing, unsubscribing and updating of subscription to
+    a newsletter that is not publicly exposed on the site
+    (has visible=False).
+
+    Runs all anonymous and user test cases with invisible newsletter.
+    """
+
+    def setUp(self):
+        super_obj = super(InvisibleAnonymousSubscribeTestCase, self)
+        super_obj.setUp()
+
+        # Make newsletter invisible
+        self.n.visible = False
+        self.n.save()
+
+
+class InvisibleUserSubscribeTestCase(UserSubscribeTestCase):
+    """
+    Tests for subscribing, unsubscribing and updating of subscription to
+    a newsletter that is not publicly exposed on the site
+    (has visible=False).
+
+    Runs all anonymous and user test cases with invisible newsletter.
+    """
+
+    def setUp(self):
+        super_obj = super(InvisibleUserSubscribeTestCase, self)
+        super_obj.setUp()
+
+        # Make newsletter invisible
+        self.n.visible = False
+        self.n.save()
+
+
+
 class ArchiveTestcase(NewsletterListTestCase):
     def setUp(self):
         """ Make sure we have a few submissions to test with. """
@@ -981,7 +1166,7 @@ class ArchiveTestcase(NewsletterListTestCase):
         # otherwise the archive will not function.
 
         (subject_template, text_template, html_template) = \
-            EmailTemplate.get_templates('message', self.newsletter)
+            self.newsletter.get_templates('message')
 
         self.assertTrue(html_template)
 
@@ -1035,18 +1220,6 @@ class ArchiveTestcase(NewsletterListTestCase):
 
         self.assertContains(response, self.submission.message.title)
 
-    def test_archive_notemplate(self):
-        """
-        The archive detail should return 404 when the template has no HTML. """
-
-        # Make sure the HTML is deleted
-        EmailTemplate.objects.update(html=None)
-
-        detail_url = self.submission.get_absolute_url()
-
-        response = self.client.get(detail_url)
-        self.assertEqual(response.status_code, 404)
-
     def test_archive_unpublished_detail(self):
         """ Assert that an unpublished submission is truly inaccessible. """
 
@@ -1063,3 +1236,93 @@ class ArchiveTestcase(NewsletterListTestCase):
 
         response = self.client.get(detail_url)
         self.assertEqual(response.status_code, 404)
+
+    @unittest.skipUnless(pytz, 'pytz could not be imported.')
+    @override_settings(
+        TIME_ZONE='Europe/Paris',
+        USE_TZ=True
+    )
+    def test_archive_timezone_regression(self):
+        """
+        Regression test for #74: Wrong submission archive urls when
+        timezones are enabled.
+
+        Ref:
+            * https://docs.djangoproject.com/en/1.5/topics/i18n/timezones/#troubleshooting
+            * https://github.com/dokterbob/django-newsletter/issues/74
+        """
+        problematic_date = datetime(2012, 3, 3, 1, 30)
+
+        paris_date = timezone.make_aware(
+            problematic_date, timezone.get_default_timezone()
+        )
+
+        # Setup submission in paris timezone
+        self.submission.publish_date = paris_date
+        self.submission.save()
+
+        # Test viewing the submission in another timezone
+        timezone.activate('America/New_York')
+
+        # Test viewing the submission
+        self.test_archive_detail()
+
+
+class ActionTemplateViewMixin(object):
+    """ Mixin for testing requests to urls for all three actions. """
+
+    def get_action_url(self, action):
+        """
+        This method should be overridden in subclasses.
+        Return url for given action.
+        """
+
+        raise NotImplementedError(
+            '%(class_name)s inherits from of ActionTemplateViewMixin '
+            'and should define get_url method.' % {
+                'class_name': self.__class__.__name__
+            }
+        )
+
+    def action_url_test(self, action):
+        """ Assertions common for all actions. """
+        response = self.client.get(self.get_action_url(action))
+
+        self.assertEquals(response.status_code, 200)
+        self.assertInContext(response, 'newsletter', Newsletter, self.n)
+        self.assertInContext(response, 'action', value=action)
+
+    def test_subscribe_url(self):
+        self.action_url_test('subscribe')
+
+    def test_unsubscribe_url(self):
+        self.action_url_test('unsubscribe')
+
+    def test_update_url(self):
+        self.action_url_test('update')
+
+
+class ActivationEmailSentUrlTestCase(
+        ActionTemplateViewMixin, SubscribeTestCase
+):
+    """
+    TestCase for testing requests to urls with activation email sent info.
+    """
+
+    def get_action_url(self, action):
+        """ Return url with email sent info for given action. """
+
+        return getattr(self, '%s_email_sent_url' % action)
+
+
+class ActionActivatedUrlTestCase(
+        ActionTemplateViewMixin, SubscribeTestCase
+):
+    """
+    TestCase for testing requests to urls with action activated info.
+    """
+
+    def get_action_url(self, action):
+        """ Return url with action activated info for given action. """
+
+        return getattr(self, '%s_activated_url' % action)
